"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SUGGESTED_EVALUATE = exports.DEFAULT_ESCAPE = exports.DEFAULT_INTERPOLATE = void 0;
var lodash_es_1 = require("lodash-es");
var merge_js_1 = require("./merge.js");
// Default lodash templates regular expressions
// https://regex101.com/r/ce27tA/5
exports.DEFAULT_INTERPOLATE = /{{{([^{}#][\s\S]+?)}}}/g;
// https://regex101.com/r/8sRC8b/8
exports.DEFAULT_ESCAPE = /{{([^{}#][\s\S]+?)}}/g;
// https://regex101.com/r/ndDqxg/4
exports.SUGGESTED_EVALUATE = /{{#([^{}].*?)}}/g;
var MessageBox = /** @class */ (function () {
    function MessageBox(_a) {
        var _b = _a === void 0 ? {} : _a, escape = _b.escape, evaluate = _b.evaluate, initialLanguage = _b.initialLanguage, interpolate = _b.interpolate, messages = _b.messages, tracker = _b.tracker;
        var _c, _d, _e;
        this.language = (_c = initialLanguage !== null && initialLanguage !== void 0 ? initialLanguage : MessageBox.language) !== null && _c !== void 0 ? _c : 'en';
        this.messageList = messages !== null && messages !== void 0 ? messages : {};
        if (tracker != null) {
            this.tracker = tracker;
            this.trackerDep = new tracker.Dependency();
        }
        // Template options
        this.interpolate = (_d = interpolate !== null && interpolate !== void 0 ? interpolate : MessageBox.interpolate) !== null && _d !== void 0 ? _d : exports.DEFAULT_INTERPOLATE;
        this.evaluate = evaluate !== null && evaluate !== void 0 ? evaluate : MessageBox.evaluate;
        this.escape = (_e = escape !== null && escape !== void 0 ? escape : MessageBox.escape) !== null && _e !== void 0 ? _e : exports.DEFAULT_ESCAPE;
    }
    MessageBox.prototype.clone = function () {
        var copy = new MessageBox({
            escape: this.escape,
            evaluate: this.evaluate,
            initialLanguage: this.language,
            interpolate: this.interpolate,
            tracker: this.tracker
        });
        copy.messages(this.messageList);
        return copy;
    };
    MessageBox.prototype.getMessages = function (language) {
        if (language == null) {
            language = this.language;
            if (this.trackerDep != null)
                this.trackerDep.depend();
        }
        var globalMessages = MessageBox.messages[language];
        var messages = this.messageList[language];
        if (messages != null) {
            if (globalMessages != null)
                messages = (0, merge_js_1.default)({}, globalMessages, messages);
        }
        else {
            messages = globalMessages;
        }
        if (messages == null)
            throw new Error("No messages found for language \"".concat(language, "\""));
        return {
            messages: messages,
            language: language
        };
    };
    MessageBox.prototype.message = function (errorInfo, _a) {
        var _b = _a === void 0 ? {} : _a, context = _b.context, language = _b.language;
        // Error objects can optionally include a preformatted message,
        // in which case we use that.
        if (typeof errorInfo.message === 'string')
            return errorInfo.message;
        var fieldName = errorInfo.name;
        var errorType = errorInfo.type;
        var genericName = MessageBox.makeNameGeneric(fieldName);
        var messages = this.getMessages(language).messages;
        var messageOrMessages = messages[errorType];
        var message;
        if (typeof messageOrMessages === 'string' || typeof messageOrMessages === 'function') {
            message = messageOrMessages;
        }
        else {
            message = genericName != null ? messageOrMessages[genericName] : undefined;
            if (message == null)
                message = messageOrMessages._default;
        }
        if (typeof message === 'string') {
            message = (0, lodash_es_1.template)(message, {
                interpolate: this.interpolate,
                evaluate: this.evaluate,
                escape: this.escape
            });
        }
        if (typeof message !== 'function')
            return "".concat(fieldName, " is invalid");
        return message(__assign(__assign({ genericName: genericName }, (context !== null && context !== void 0 ? context : {})), errorInfo));
    };
    MessageBox.prototype.messages = function (messages) {
        (0, merge_js_1.default)(this.messageList, messages);
    };
    MessageBox.prototype.setLanguage = function (language) {
        this.language = language;
        if (this.trackerDep != null)
            this.trackerDep.changed();
    };
    MessageBox.makeNameGeneric = function (name) {
        if (typeof name !== 'string')
            return null;
        return name.replace(/\.[0-9]+(?=\.|$)/g, '.$');
    };
    MessageBox.defaults = function (_a) {
        var _b = _a === void 0 ? {} : _a, escape = _b.escape, evaluate = _b.evaluate, initialLanguage = _b.initialLanguage, interpolate = _b.interpolate, messages = _b.messages;
        if (typeof initialLanguage === 'string')
            MessageBox.language = initialLanguage;
        if (interpolate instanceof RegExp)
            MessageBox.interpolate = interpolate;
        if (evaluate instanceof RegExp)
            MessageBox.evaluate = evaluate;
        if (escape instanceof RegExp)
            MessageBox.escape = escape;
        if (messages != null) {
            if (MessageBox.messages == null)
                MessageBox.messages = {};
            (0, merge_js_1.default)(MessageBox.messages, messages);
        }
    };
    MessageBox.messages = {};
    return MessageBox;
}());
exports.default = MessageBox;
