"use strict";
/**
 * We have relatively simple deep merging requirements in this package.
 * We are only ever merging messages config, so we know the structure,
 * we know there are no arrays, and we know there are no constructors
 * or weirdly defined properties.
 *
 * Thus, we can write a very simplistic deep merge function to avoid
 * pulling in a large dependency.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = merge;
function merge(destination) {
    var sources = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        sources[_i - 1] = arguments[_i];
    }
    sources.forEach(function (source) {
        Object.keys(source).forEach(function (prop) {
            var _a, _b;
            if (((_a = source[prop]) === null || _a === void 0 ? void 0 : _a.constructor) === Object) {
                if (((_b = destination[prop]) === null || _b === void 0 ? void 0 : _b.constructor) !== Object) {
                    var temp = {};
                    destination[prop] = temp;
                }
                merge(destination[prop], source[prop]);
            }
            else {
                destination[prop] = source[prop];
            }
        });
    });
    return destination;
}
